package be.iminds.ilabt.jfed.ui.javafx.automated_testing_gui;

import be.iminds.ilabt.jfed.call_log_output.SerializableApiCallDetailsFactory;
import be.iminds.ilabt.jfed.fedmon.webapi.service.json.Server;
import be.iminds.ilabt.jfed.gui_preferences.HLPreferenceKey;
import be.iminds.ilabt.jfed.highlevel.history.ApiCallHistory;
import be.iminds.ilabt.jfed.log.Logger;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUserProvider;
import be.iminds.ilabt.jfed.lowlevel.testbed_info.TestbedInfoSource;
import be.iminds.ilabt.jfed.gui_preferences.JFedHLPreferences;
import be.iminds.ilabt.jfed.testing.TestClassList;
import be.iminds.ilabt.jfed.testing.base.*;
import be.iminds.ilabt.jfed.ui.javafx.choosers.ServerChooser;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import be.iminds.ilabt.jfed.ui.javafx.log_gui.LogHistoryPanel;
import be.iminds.ilabt.jfed.ui.javafx.probe_gui.command_arguments.*;
import be.iminds.ilabt.jfed.util.common.IOUtils;
import be.iminds.ilabt.jfed.util.library.JFedUtils;
import be.iminds.ilabt.jfed.lowlevel.authority.legacy.TargetAuthority;
import javafx.application.Platform;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.concurrent.Task;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;
import javafx.scene.web.WebEngine;
import javafx.scene.web.WebView;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import org.slf4j.LoggerFactory;

import javax.inject.Inject;
import javax.inject.Provider;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * ProbeController
 */
public class AutomatedTester {
    private static final org.slf4j.Logger LOG = LoggerFactory.getLogger(AutomatedTester.class);
    Task<ApiTestResult> runTask;

    @FXML
    private ServerChooser targetAuthChooserController;
    @FXML
    private Label userAuthLabel;
    @FXML
    private ListView<String> testClassesListView;
    @FXML
    private ListView<String> testGroupsListView;
    @FXML
    private Button runButton;
    @FXML
    private ScrollPane argumentsScrollPane;
//    @FXML
//    private ScrollPane rawArgumentsScrollPane;
    @FXML
    private TextArea rawArgumentsTextArea;
    @FXML
    private GridPane argumentsBox;
    @FXML
    private WebView webView;
    @FXML
    private Parent progressBarBox;
    @FXML
    private Label progressLabel;
    @FXML
    private ProgressBar progressBar;
    @FXML
    private TabPane testReportTabPane;
    @FXML
    private Tab testReportTab;

    private ObservableList<String> testGroups = FXCollections.observableArrayList();
    private Class<? extends ApiTest> testClass;
    private Map<ApiTestConfigEditor.ApiTestConfigParameter, CommandArgumentChooser> extraArguments = new HashMap<>();
    private Map<ApiTestConfigEditor.ApiTestConfigParameter, CheckBox> extraArgumentsNullBoxes = new HashMap<>();
    private Stage historyStage;

    private final GeniUserProvider geniUserProvider;
    private final ApiCallHistory apiCallHistory;
    private final TestbedInfoSource testbedInfoSource;
    private final Logger logger;
    private final Provider<LogHistoryPanel> logHistoryPanelProvider;
    private final JFedHLPreferences jFedPreferences;
    private final ApiTestFactory apiTestFactory;
    private final SerializableApiCallDetailsFactory serializableApiCallDetailsFactory;

    private ApiTestMetaData apiTestMetaData;
    private ApiTestConfigEditor configEditor;

    private AutomatedTesterModel model;

    @Inject
    public AutomatedTester(GeniUserProvider geniUserProvider, ApiCallHistory apiCallHistory,
                           TestbedInfoSource testbedInfoSource, Logger logger, Provider<LogHistoryPanel> logHistoryPanelProvider,
                           JFedHLPreferences jFedPreferences,
                           ApiTestFactory apiTestFactory,
                           SerializableApiCallDetailsFactory serializableApiCallDetailsFactory) {
        this.geniUserProvider = geniUserProvider;
        this.apiCallHistory = apiCallHistory;
        this.testbedInfoSource = testbedInfoSource;
        this.logger = logger;
        this.logHistoryPanelProvider = logHistoryPanelProvider;
        this.jFedPreferences = jFedPreferences;
        this.apiTestFactory = apiTestFactory;
        this.serializableApiCallDetailsFactory = serializableApiCallDetailsFactory;
    }

    @FXML
    private void initialize() {
        ObservableList<String> testClasses = FXCollections.observableArrayList(allTestClasses());
        testClassesListView.setItems(testClasses);
        testGroupsListView.setItems(testGroups);

        this.model = new AutomatedTesterModel(
                testClassesListView.getSelectionModel(),
                testGroupsListView.getSelectionModel(),
                targetAuthChooserController,
                new SimpleBooleanProperty(false));

        model.selectedClassNameProperty().addListener(
                (observableValue, oldSelectedItem, newSelectedItem) ->
                        updateSelectedClass(newSelectedItem));

        progressBarBox.managedProperty().bind(progressBarBox.visibleProperty());
        progressBarBox.setVisible(false);

        model.restore();
        updateSelectedClass(model.getSelectedClassName());

        rawArgumentsTextArea.visibleProperty().bind(model.showRawModelProperty());
        argumentsScrollPane.visibleProperty().bind(model.showRawModelProperty().not());

        LOG.debug("at start showRaw="+model.showRawModelProperty().getValue());

        assert testReportTabPane != null;
        assert testReportTab != null;
    }


    public static List<String> allTestClasses() {
        return TestClassList.getInstance().allTestClasses();
    }

    public void updateSelectedClass(String selectedClassName) {
        if (selectedClassName == null) {
            testClass = null;
            testGroups.clear();
            return;
        }

        testClass = null;
        try {
            testClass = Class.forName(selectedClassName).asSubclass(ApiTest.class);
            assert ApiTest.class.isAssignableFrom(testClass) : testClass.getName() + " is not a ApiTest class";
        } catch (ClassNotFoundException e) {
            throw new RuntimeException("Bug in Test GUI: could not find test class " + selectedClassName, e);
        }
        assert testClass != null;

        testGroups.setAll(testGroups(testClass));

        this.apiTestMetaData = ApiTest.getMetaData(testClass);
        this.configEditor = apiTestMetaData.getApiTestConfigEditor();
        List<ApiTestConfigEditor.ApiTestConfigParameter> parameters = configEditor.getParameters();

        extraArguments.clear();
        extraArgumentsNullBoxes.clear();
        argumentsBox.getChildren().clear();

        int row = 0;

        if (!parameters.isEmpty()) {
            Label head = new Label("Extra Test Arguments: ");
            VBox.setMargin(head, new Insets(10.0/*top*/, 0.0/*right*/, 0.0/*bottom*/, 0.0/*left*/));
            argumentsBox.getChildren().add(head);

            for (ApiTestConfigEditor.ApiTestConfigParameter parameter : parameters) {
                System.out.println("Parameter="+parameter);
//                String key = parameter.getLastName();
                String key = parameter.getName().stream().collect(Collectors.joining("."));
                boolean required = parameter.isAllowNull();

                CommandArgumentChooser commandArgumentChooser_helper = null;
                if (parameter.getType().equals(String.class)) {
                    String defaultValue = parameter.getDefaultValue() == null ? "" : parameter.getDefaultValue()+"";
                    if (key.equals("fixed_rspec") || key.trim().toLowerCase().endsWith("credential")) {
                        commandArgumentChooser_helper = new MultiLineStringArgumentChooser(defaultValue, true);
                    } else
                        commandArgumentChooser_helper = new StringArgumentChooser(defaultValue);
                }
                if (parameter.getType().equals(Boolean.class) || parameter.getType().equals(boolean.class)) {
                    Boolean defaultValue = parameter.getDefaultValue() == null ? false : (Boolean) parameter.getDefaultValue();
                    commandArgumentChooser_helper = new BooleanArgumentChooser(defaultValue);
                }
                if (parameter.getType().equals(Integer.class) || parameter.getType().equals(int.class)) {
                    Integer defaultValue = parameter.getDefaultValue() == null ? 0 : (Integer) parameter.getDefaultValue();
                    commandArgumentChooser_helper = new IntegerArgumentChooser(defaultValue);
                }
                if (parameter.getType().equals(Long.class) || parameter.getType().equals(long.class)) {
                    Long defaultValue = parameter.getDefaultValue() == null ? 0 : (Long) parameter.getDefaultValue();
                    commandArgumentChooser_helper = new LongArgumentChooser(defaultValue);
                }
                if (parameter.getType().isEnum()) {
                    Enum defaultValue = parameter.getDefaultValue() == null ? null : (Enum) parameter.getDefaultValue();
                    commandArgumentChooser_helper = new EnumArgumentChooser(parameter.getType(), defaultValue);
                }
                final CommandArgumentChooser commandArgumentChooser = commandArgumentChooser_helper;
                if (commandArgumentChooser == null) {
                    LOG.warn("Cannot show edit for option "+parameter.getName()+" of type "+parameter.getType().getName());
                    continue;
                }
                if (parameter.isAllowNull()) {
                    CheckBox nullBox = new CheckBox("");
                    GridPane.setRowIndex(nullBox, row);
                    GridPane.setColumnIndex(nullBox, 0);
                    argumentsBox.getChildren().add(nullBox);
                    nullBox.setSelected(parameter.getDefaultValue() != null);
                    extraArgumentsNullBoxes.put(parameter, nullBox);
                    commandArgumentChooser.disableProperty().bind(nullBox.selectedProperty().not());
                }

                Label parameterLabel = new Label(key + ":");
                GridPane.setRowIndex(parameterLabel, row);
                GridPane.setColumnIndex(parameterLabel, 1);
                argumentsBox.getChildren().add(parameterLabel);

                HBox hBox = new HBox();
                HBox.setMargin(parameterLabel, new Insets(0.0/*top*/, 10.0/*right*/, 0.0/*bottom*/, 0.0/*left*/));
                HBox.setHgrow(commandArgumentChooser, Priority.ALWAYS);
                hBox.getChildren().addAll(commandArgumentChooser);
                VBox.setMargin(hBox, new Insets(10.0/*top*/, 0.0/*right*/, 0.0/*bottom*/, 10.0/*left*/));

                GridPane.setRowIndex(hBox, row);
                GridPane.setColumnIndex(hBox, 2);
                GridPane.setColumnSpan(hBox, 2);
                argumentsBox.getChildren().add(hBox);

                if (key.endsWith("_file")) {
                    Button b = new Button("Choose File...");
                    hBox.getChildren().add(b);
                    b.setOnAction(actionEvent -> {
                        FileChooser fc = new FileChooser();

                        //set the default open dir
                        if (System.getProperty("user.home") != null)
                            fc.setInitialDirectory(new File(System.getProperty("user.home")));

                        if (jFedPreferences.containsPreference(HLPreferenceKey.PREF_RSPECFOLDER_REQUEST)) {
                            File dir =
                                    new File(jFedPreferences.getString(HLPreferenceKey.PREF_RSPECFOLDER_REQUEST));
                            if (dir.exists()) {
                                fc.setInitialDirectory(dir);
                            }
                        }

                        File file = fc.showOpenDialog(null);

                        if (file == null)
                            return;

                        ((StringArgumentChooser) commandArgumentChooser).setValue(file.getAbsolutePath());
                    });
                }

                row++;
                extraArguments.put(parameter, commandArgumentChooser);
            }
        }

        if (runTask == null)
            progressBarBox.setVisible(false);
    }

    public List<String> testGroups(Class<? extends ApiTest> testClass) {
        List<String> res = Arrays.stream(testClass.getMethods())
                .flatMap(m -> Arrays.stream(m.getDeclaredAnnotations()))
                .filter(annotation -> annotation instanceof ApiTest.Test)
                .map(ApiTest.Test.class::cast)
                .flatMap(testAnnotation -> Arrays.stream(testAnnotation.groups()))
                .distinct()
                .collect(Collectors.toList());

        res.add("<no group>");
        return res;
    }

    @FXML
    private void runTests() {
        if (runTask == null)
            progressBarBox.setVisible(false);

        if (runTask != null) {
            JFDialogs.create()
                    .owner(runButton.getScene().getWindow())
                    .message("Already running tests. Wait until current tests are finished.")
                    .showError();
            return;
        }
        if (!geniUserProvider.isUserLoggedIn()) {
            JFDialogs.create()
                    .owner(runButton.getScene().getWindow())
                    .message("Can only run tests if user is logged in: log in first")
                    .showError();
            return;
        }
        if (targetAuthChooserController.getSelectedServer() == null) {
            JFDialogs.create()
                    .owner(runButton.getScene().getWindow())
                    .message("No target authority is selected")
                    .showError();
            return;
        }
        if (model.getSelectedClassName() == null || testClass == null) {
            JFDialogs.create()
                    .owner(runButton.getScene().getWindow())
                    .message("No test is selected")
                    .showError();
            return;
        }
        assert testClass != null;

        GeniUser user = geniUserProvider.getLoggedInGeniUser();

        Server authForRspec = targetAuthChooserController.getSelectedServer();

        //do not auto select needed auth like this:
//        Server authToConnect = authorityFinder.findByUrn(
//                targetAuthChooserController.getSelectedServer().getUrn(),
//                AuthorityFinder.Purpose.CREATE_SLIVER);
        //bacuase user chooses auth from list, so we will contact only explicitly requested auth
        Server authToConnect = targetAuthChooserController.getSelectedServer();

        TargetAuthority testedAuthority = new TargetAuthority(targetAuthChooserController.getSelectedServer().getDefaultComponentManagerAsGeniUrn(), authForRspec, authToConnect);

        String group = model.getSelectedTestGroup();
        if (Objects.equals(group, "<no group>"))
            group = null;
        final String testGroup = group;

        final String configString = getCurrentConfigString();

        LOG.debug("Running Test: {} group={}", testClass.getName(), group);

        runTask = new Task<ApiTestResult>() {
            @Override
            protected ApiTestResult call() throws Exception {
                LOG.debug("   Starting Tests");
                updateMessage("Starting Tests");
                updateProgress(0, 100);

                try {
                    Platform.runLater(() -> {
                        assert testReportTabPane != null;
                        assert testReportTab != null;
                        testReportTabPane.getTabs().retainAll(testReportTab);
                        shownResult = null;
                        shownHtmlOutput = null;
                        WebEngine webEngine = webView.getEngine();
                        webEngine.loadContent("<html><body><p>Running test...</p></body></html>");
                    });

                    ApiTest test = apiTestFactory.createApiTest(logger, testedAuthority, testClass, configString);

                    AutomatedTestRunner automatedTestRunner = new AutomatedTestRunner(testedAuthority, serializableApiCallDetailsFactory);
                    shownResult =
                            automatedTestRunner.runTest(test, testGroup, new AutomatedTestRunner.TestListener() {
                                @Override
                                public void onStart(String testname, int testNr, int testCount) {
                                    LOG.debug("Running test: {} ...", testname);
                                    updateMessage("Running test: " + testname);
                                    updateProgress(testNr - 1, testCount);
                                }

                                @Override
                                public void onResult(ApiTestResult.ApiTestMethodResult result, int testNr, int testCount) {
                                    LOG.debug("     Test result: {}", result);
                                    updateProgress(testNr, testCount);
                                }

                                @Override
                                public void onAllTestDone(ApiTestResult result, int testCount) {
                                    updateProgress(testCount, testCount);
                                    updateMessage("All tests completed");
                                }
                            }, false);

                    updateMessage("Test Finished. Generating HTML report...");
                    LOG.debug("Test Finished. Generating HTML report...");

                    try {
                        shownHtmlOutput = shownResult.toHtmlString(true, true); //webengine needs polyfills
                    } catch (AssertionError | Exception e) {
                        LOG.error("Error while generating HTML: ", e);
                        shownHtmlOutput = "Error while generating HTML: "+e.getMessage();
                    }
                    updateMessage("Test Finished. HTML report created.");
                    LOG.debug("Test Finished. HTML report created.");

                    Platform.runLater(() -> {
                        WebEngine webEngine = webView.getEngine();
                        webEngine.loadContent(shownHtmlOutput, "text/html");
//                        executePolyFills(webEngine);
                        LOG.debug("   Showing test");
                    });

                    for (ApiTestResult.ApiTestMethodResult testMethodResult: shownResult.getResultList()) {
                        for (ApiTestResult.ApiTestMethodResult.FedmonResultExtra extra : testMethodResult.getFedmonResultExtras()) {
                            TextArea extraTextArea = new TextArea();
                            extraTextArea.setEditable(false);
                            extraTextArea.setText(extra.getValue()+"");
                            Tab extraTab = new Tab(extra.getNestedKey().stream().collect(Collectors.joining(".")));
                            extraTab.setContent(extraTextArea);
                            Platform.runLater(() -> {
                                testReportTabPane.getTabs().add(extraTab);
                            });
                        }
                    }

                    runTask = null;
                    return shownResult;
                } catch (AssertionError e) {
                    LOG.error("Something went wrong while running tests: " + e, e);
                    updateMessage("AssertionError has occured while running test.");
                    runTask = null;
                    return null;
                } catch (Exception e) {
                    LOG.error("Something went wrong while running tests: " + e, e);
                    updateMessage("Exception has occured while running test.");
                    runTask = null;
                    return null;
                }
            }
        };
        progressBar.progressProperty().bind(runTask.progressProperty());
        progressLabel.textProperty().bind(runTask.messageProperty());
        progressBarBox.setVisible(true);
        new Thread(runTask).start();
    }

    private void saveCurrentConfigToEditor() {
        for (Map.Entry<ApiTestConfigEditor.ApiTestConfigParameter, CommandArgumentChooser> e : extraArguments.entrySet()) {
            ApiTestConfigEditor.ApiTestConfigParameter parameter = e.getKey();
            CommandArgumentChooser chooser = e.getValue();

            CheckBox nullBox = extraArgumentsNullBoxes.get(parameter);
            if (nullBox != null && !nullBox.isSelected()) {
                configEditor.setParameter(parameter, null);
            } else {
                if (chooser.valueProperty() != null) {
                    if (chooser.valueProperty().getValue() != null) {
                        if (chooser.valueProperty().getValue() instanceof String) {
                            if (!Objects.equals(((String) chooser.valueProperty().getValue()).trim(), ""))
                                configEditor.setParameter(parameter, chooser.valueProperty().getValue());
                        } else {
                            configEditor.setParameter(parameter, chooser.valueProperty().getValue());
                        }
                    }
                }
            }
        }
    }

    private String getCurrentConfigString() {
        if (rawArgumentsTextArea.isVisible()) {
            return rawArgumentsTextArea.getText();
        } else {
            saveCurrentConfigToEditor();

            return configEditor.build().stringValue();
        }
    }

    private String shownHtmlOutput = null;
    private ApiTestResult shownResult = null;
    public void saveHtml() {
        if (shownHtmlOutput == null) return;

        FileChooser fc = new FileChooser();
        //set the default open dir
        if (System.getProperty("user.home") != null)
            fc.setInitialDirectory(new File(System.getProperty("user.home")));
        fc.setInitialFileName("result-" + testClass.getName() + ".html");
        File file = fc.showSaveDialog(null);

        if (file == null)
            return;

        try {
            IOUtils.stringToFile(file, shownHtmlOutput);
        } catch (Exception e) {
            LOG.error("Error while writing HTML to file: ", e);
            throw e;
        }
    }
    public void saveXml() {
        if (shownResult == null) return;

        FileChooser fc = new FileChooser();
        //set the default open dir
        if (System.getProperty("user.home") != null)
            fc.setInitialDirectory(new File(System.getProperty("user.home")));
        fc.setInitialFileName("result-overview-" + testClass.getName() + ".xml");
        File file = fc.showSaveDialog(null);

        if (file == null)
            return;

        try {
            shownResult.toXmlOverview(file, true/*showSecurityRisk*/);
        } catch (Exception e) {
            LOG.error("Error while generating XML overview: ", e);
            throw e;
        }
    }

    @FXML
    private void showLogHistory() {
        if (historyStage == null) {
            try {
                LogHistoryPanel logHistoryPanel = logHistoryPanelProvider.get();
                logHistoryPanel.setApiCallHistory(apiCallHistory);

                historyStage = new Stage();
                historyStage.setTitle("Call Log History");
                Scene scene = new Scene(logHistoryPanel);
                historyStage.setScene(scene);
//                historyStage.sizeToScene();
                historyStage.setWidth(800.0);
                historyStage.setHeight(600.0);
            } catch (Exception e) {
                LOG.error("Error while loading Call Log History window", e);
            }
        }

        assert historyStage != null;

        historyStage.show();
    }


    public void resetArguments() {
        for (Map.Entry<ApiTestConfigEditor.ApiTestConfigParameter, CommandArgumentChooser> e : extraArguments.entrySet()) {
            ApiTestConfigEditor.ApiTestConfigParameter parameter = e.getKey();
            CommandArgumentChooser chooser = e.getValue();

            CheckBox nullBox = extraArgumentsNullBoxes.get(parameter);
            if (nullBox != null) {
                nullBox.setSelected(parameter.getDefaultValue() != null);
            }

            if (chooser instanceof StringArgumentChooser || chooser instanceof MultiLineStringArgumentChooser) {
                String defaultValue = parameter.getDefaultValue() == null ? "" : parameter.getDefaultValue() + "";
                if (chooser instanceof StringArgumentChooser)
                    ((StringArgumentChooser) chooser).setValue(defaultValue);
                if (chooser instanceof MultiLineStringArgumentChooser)
                    ((MultiLineStringArgumentChooser) chooser).setValue(defaultValue);
            }
            if (chooser instanceof BooleanArgumentChooser) {
                Boolean defaultValue = parameter.getDefaultValue() == null ? false : (Boolean) parameter.getDefaultValue();
                ((BooleanArgumentChooser) chooser).setValue(defaultValue);
            }
            if (chooser instanceof IntegerArgumentChooser) {
                Integer defaultValue = parameter.getDefaultValue() == null ? 0 : (Integer) parameter.getDefaultValue();
                ((IntegerArgumentChooser) chooser).setValue(defaultValue);
            }

        }
    }
    private File AUTOMATED_TESTER_STORED_ARGUMENTS_FILE = new File(JFedUtils.getUserDataDirectoryFile(), "saved_automated_tester_arguments");
    public void saveArguments() throws IOException {
        FileWriter w = new FileWriter(AUTOMATED_TESTER_STORED_ARGUMENTS_FILE);
        w.write(getCurrentConfigString());
        w.close();
    }
    public void loadArguments() throws IOException {
        String configString = IOUtils.fileToString(AUTOMATED_TESTER_STORED_ARGUMENTS_FILE);
        rawArgumentsTextArea.setText(configString);
        loadArgumentsFromString(configString);
    }
    public void loadArgumentsFromString(String configString) {
        ApiTestConfig apiTestConfig = apiTestMetaData.parseApiTestConfig(configString);
        configEditor.setConfig(apiTestConfig);

        for (Map.Entry<ApiTestConfigEditor.ApiTestConfigParameter, CommandArgumentChooser> e : extraArguments.entrySet()) {
            ApiTestConfigEditor.ApiTestConfigParameter parameter = e.getKey();

            CheckBox nullBox = extraArgumentsNullBoxes.get(parameter);

            Object newValue = configEditor.getParameter(parameter);
            if (nullBox != null) {
                nullBox.setSelected(newValue != null);
            }

            CommandArgumentChooser chooser = e.getValue();
            if (chooser instanceof StringArgumentChooser) {
                if (newValue == null)
                    newValue = "";
                ((StringArgumentChooser) chooser).setValue(newValue + "");
            }
            if (chooser instanceof MultiLineStringArgumentChooser) {
                if (newValue == null)
                    newValue = "";
                ((MultiLineStringArgumentChooser) chooser).setValue(newValue + "");
            }
            if (chooser instanceof IntegerArgumentChooser && newValue instanceof Integer)
                ((IntegerArgumentChooser) chooser).setValue((Integer)newValue);
            if (chooser instanceof LongArgumentChooser && newValue instanceof Long)
                ((LongArgumentChooser) chooser).setValue((Long)newValue);
            if (chooser instanceof BooleanArgumentChooser && newValue instanceof Boolean)
                ((BooleanArgumentChooser) chooser).setValue((Boolean)newValue);
            if (chooser instanceof EnumArgumentChooser && newValue instanceof String)
                ((EnumArgumentChooser) chooser).setStringValue((String) newValue);
        }
    }

    @FXML
    public void swapRaw() {
        boolean showRaw = model.toggleShowRaw();
        LOG.debug("New showRaw="+showRaw);

        if (showRaw) {
            rawArgumentsTextArea.setText(getCurrentConfigString());
        } else {
            loadArgumentsFromString(rawArgumentsTextArea.getText());
        }
    }
}
